/*
libclamavmodule.c, version 0.3.1

(c) 2003 Jan ONDREJ (SAL) <ondrejj@salstar.sk>
 
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

*/

#include <Python.h>
#include <clamav.h>

struct cl_node *root = NULL;
struct cl_limits limits;

static PyObject *
libclamav_loaddbdir(self)
	PyObject *self;
{
	int ret,virnum;
	if (root != NULL) return Py_BuildValue("i", -1);
	ret=cl_loaddbdir(cl_retdbdir(), &root, &virnum);
	if (ret) {
	  PyErr_SetString(PyExc_RuntimeError, cl_perror(ret));
	  return NULL;
	}
	if (!root) {
	  PyErr_SetString(PyExc_RuntimeError,
	    "Database initialization error.\n");
	  return NULL;
	}
	cl_buildtrie(root);
        return Py_BuildValue("i", virnum);
}

static PyObject *
libclamav_set_limits(self,args)
	PyObject *self;
	PyObject *args;
{
	int maxfiles,maxfilesize,maxreclevel,ok;
	ok = PyArg_ParseTuple(args, "iii", &maxfiles, &maxfilesize, &maxreclevel);
	limits.maxfiles=maxfiles;
	limits.maxfilesize=maxfilesize;
	limits.maxreclevel=maxreclevel;
	return Py_None;
}

static PyObject *
libclamav_scandesc(self,args)
	PyObject *self;
	PyObject *args;
{
	int ret,fd;
	unsigned long int size;
	char *virname;

	PyArg_ParseTuple(args, "i", &fd);
	if ((ret=cl_scandesc(fd, &virname, &size, root, &limits, CL_ARCHIVE|CL_MAIL))
	   == CL_VIRUS) {
		return Py_BuildValue("s", virname);
	} else if (ret == CL_CLEAN) {
		return Py_BuildValue("s", "");
	}
	PyErr_SetString(PyExc_RuntimeError, cl_perror(ret));
	return NULL;
}

static PyObject *
libclamav_scanbuff(self,args)
	PyObject *self;
	PyObject *args;
{
	int ret, size;
	unsigned int bs;
	char *virname, *buf;

	PyArg_ParseTuple(args, "t#", &buf, &bs);
	if ((ret=cl_scanbuff(buf, bs, &virname, root)) == CL_VIRUS) {
		return Py_BuildValue("s", virname);
	} else if (ret == CL_CLEAN) {
		return Py_BuildValue("s", "");
	}
	PyErr_SetString(PyExc_RuntimeError, cl_perror(ret));
	return NULL;
}

static PyObject *
libclamav_freetrie(self)
	PyObject *self;
{
	cl_freetrie(root);
	root=NULL;
	return Py_None;
}

static PyMethodDef ClamAVMethods[] = {
  {"loaddbdir", libclamav_loaddbdir, METH_VARARGS, "load db dir"},
  {"set_limits", libclamav_set_limits, METH_VARARGS, "set limits"},
  {"scandesc", libclamav_scandesc, METH_VARARGS, "scan fd"},
  {"scanbuff", libclamav_scanbuff, METH_VARARGS, "scan buffer"},
  {"freetrie", libclamav_freetrie, METH_VARARGS, "free trie"},
  {NULL, NULL, 0, NULL}
};

void initlibclamav(void)
{
	PyObject *m;
	m=Py_InitModule("libclamav", ClamAVMethods);
	limits.maxfiles=1000;
	limits.maxfilesize=10 * 1048576;
	limits.maxreclevel=8;
}
